;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defgeneric nrealize (compo &rest restargs &key context depth &allow-other-keys)
  (:documentation

"Generic Function
   nrealize (compo &rest restargs &key :context :depth &allow-other-keys)
Arguments
   A composition to be realized. 
   restargs is provided as a &rest argument list to give the methods body an access to
   the entire list of the given key arguments.
   The :context key argument is the context to use in the realization process.
   The :depth key argument should be an integer if provided. If a depth is provided, the
   realization will be a granulation at the specified depth (see the granulation and the
   recursion context-items).
Values
   Always returns a valid composition.
Purpose
   This function is callable from anywhere by any client.
   It is specialized on the class 'compo' and can be overriden by any subclass of
   'compo'.
   From the caller's point of view, it can be seen as the destructive version of the
   'realize' function, saving the cost of a complete cloning of the composition, when 
   there is no need to keep the original unchanged." ) )

(defmethod nrealize ((compo abstract-compo)
                    &rest restargs
                    &key (context (context compo))
                         depth )

"Primary method
   nrealize specialized on compo
Behaviour
   If the :context key argument is not provided, it defaults to the current value of the
   'default-context' class slot in the composition to be realized.
   A new instance of the 'realization' context item whith a clone of the context as a
   :context initarg (see the 'realization' context item) is obtained.
   The 'realize-using-context-item' generic-function is applied to this new instance with
   the composition and restargs as a remaining argument list. Thus, the realization
   process itself is reflexively delegated by realize to the 'realization' context-item.
   It is useful to make the realization process itself a context-item, since this allow
   it to be recursively applied to subcompositions (see the 'recursion' context-item).
   Finally, a constant version of the realized composition (see the 'make-constant'
   generic function) is obtained and returned as a result."

  (declare (ignore depth))
  (apply #'realize-using-context-item
         (make-instance 'realization
           :context (clone context) )
         compo
         restargs )
  (make-constant compo context) )