;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defun pprint-compo (stream self)

"Function
   pprint-compo (stream self)
Arguments
   The output stream in which the composition is to be printed.
   The composition to be printed.
Values
   No value is returned.
   As a side effect, the composition is printed to the output stream.
Purpose
   This function is installed in the dispatch table used by the standard Common Lisp function
   pprint, for the pretty printing of compositions.
Behaviour
   An opening parenthesis starts the printing of the composition, and a closing
   parenthesis ends it.
   pprint-compo prints the class name, delegates the printing of the eventual
   properties defined by any subclass of 'compo' to the generic function
   'specialized-print' and the printing of each subcomposition to the function
   'sub-pprint', taking care of the two following special cases :
   . If the value of the slot 'subs' in the composition is not a list, it
     represents the particular case of an atomic subcomposition, which is indicated
     by making its representation preceded by the keyword :subs. Note that this is
     also the way to specify this particular case when entering a new composition
     (see normalize-compo).
   . In that case of an atomic subcomposition, if the subcomposition represents
     the composition itself, it is represented by the keyword :self to avoid any
     circularity when printing.
   An indentation occurs before the printing of the subcompositions."

  (pprint-logical-block (stream nil)
    (write-string "(" stream)
    (write (class-name (class-of self)) :stream stream)
    (specialized-print self stream t)
    (when (subs self)
      (write-string " " stream)
      (if (eq (subs self) self)
        (write :self :stream stream)
        (if (atom (subs self))
          (progn
            (write-string ":SUBS " stream)
            (sub-pprint (subs self) self stream) )
          (progn
            (pprint-indent :block 3 stream)
            (mapl #'(lambda (x)
                      (pprint-newline :mandatory stream)
                      (sub-pprint (car x) self stream) )
                  (subs self) )
            (pprint-indent :block 0 stream) ) ) ) )
    (write-string ")" stream) ) )

; 'pprint' not being implemented in CLisp, 'set-pprint-dispatch' is tested to be
; present or not before being called.

(and (fboundp 'set-pprint-dispatch)
     (set-pprint-dispatch 'abstract-compo
                          #'pprint-compo ) )
