;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defgeneric sub-pprint (sub super stream)
  (:documentation

"Generic Function
   sub-pprint (sub super stream)
Arguments
   A subcomposition to be pretty printed.
   The name of the class of the composition that is the root of the structure currently
   being pretty printed.
   The current output stream.
Values
   No value is returned.
   As a side effect, the subcomposition is pretty printed in the given output stream." ) )

(defmethod sub-pprint (sub super stream)

"Primary method
   defaulting sub-pprint
Behaviour
   The subcomposition is basically writed (using write) in the output stream, included
   in its own logical block"

  (declare (ignore super))
  (pprint-logical-block (stream nil)
    (write sub :stream stream) ) )

(defmethod sub-pprint ((sub abstract-compo) super stream)

"Primary method
   sub-pprint specialized on 'compo'
Behaviour
   An opening parenthesis starts the printing of the subcomposition, and a closing
   parenthesis ends it.
   sub-pprint prints the class name only if its differs from the class name of the root,
   delegates the printing of the eventual properties defined by any subclass of 'compo'
   to the generic function 'specialized-print' and recursively applies sub-pprint for
   each subcomposition, taking care of the two following special cases :
   . If the value of the slot 'subs' in the composition is not a list, it represents the
     particular case of an atomic subcomposition, which is indicated by making its
     representation preceded by the keyword :subs.
   . In that case of an atomic subcomposition, if the subcomposition represents the
     composition itself, it is represented by the keyword :self to avoid any circularity
     when printing.
   An indentation occurs before the printing of the subcompositions."

  (pprint-logical-block (stream nil)
    (write-string "(" stream)
    (let ((sep))
      (unless (eq (class-name (class-of sub))
                  (class-name (class-of super)) )
        (write (class-name (class-of sub)) :stream stream)
        (setf sep t) )
      (setf sep (specialized-print sub stream sep))
      (when (subs sub)
        (when sep (write-string " " stream))
        (if (eq (subs sub) sub)
          (write :self :stream stream)
          (if (atom (subs sub))
            (progn
              (write-string ":SUBS " stream)
              (sub-print (subs sub) super stream) )
            (progn
              (pprint-indent :block 3 stream)
              (mapl #'(lambda (x)
                        (pprint-newline :mandatory stream)
                        (sub-pprint (car x) super stream))
                    (subs sub) )
              (pprint-indent :block 0 stream) ) ) ) )
      (write-string ")" stream) ) ) )